/*******************************************************************************
 * Copyright (c) 2009 IBM Corporation and others.
 * All rights reserved. This program and the accompanying materials
 * are made available under the terms of the Eclipse Public License v1.0
 * which accompanies this distribution, and is available at
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Contributors:
 *     IBM Corporation - initial API and implementation
 *******************************************************************************/
package org.eclipse.e4.languages.javascript.jsdi.rhino;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.Map;

import org.eclipse.e4.languages.javascript.debug.connect.JSONConstants;
import org.eclipse.e4.languages.javascript.jsdi.PropertiesReference;
import org.eclipse.e4.languages.javascript.jsdi.Property;
import org.eclipse.e4.languages.javascript.jsdi.Value;

/**
 * Rhino implementation of {@link PropertiesReference}
 * 
 * @since 1.0
 */
abstract public class PropertiesReferenceImpl extends MirrorImpl implements
		PropertiesReference {

	protected StackFrameImpl frame;
	private final List properties = new ArrayList();

	/**
	 * Constructor
	 * 
	 * @param vm
	 * @param body
	 * @param frame
	 */
	public PropertiesReferenceImpl(VirtualMachineImpl vm, Map body,
			StackFrameImpl frame) {
		super(vm);
		this.frame = frame;
		List jsonProperties = (List) body.get(JSONConstants.PROPERTIES);
		for (Iterator iterator = jsonProperties.iterator(); iterator.hasNext();) {
			Map jsonProperty = (Map) iterator.next();
			String name = (String) jsonProperty.get(JSONConstants.NAME)
					.toString();
			Number ref = (Number) jsonProperty.get(JSONConstants.REF);
			PropertyImpl property = new PropertyImpl(vm, name, ref);
			properties.add(property);
		}
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.e4.languages.javascript.jsdi.PropertiesReference#getValue
	 * (org.eclipse.e4.languages.javascript.jsdi.Property)
	 */
	public Value getValue(Property property) {
		PropertyImpl propertyImpl = (PropertyImpl) property;
		return frame.lookupValue(propertyImpl.getRef());
	}

	/*
	 * (non-Javadoc)
	 * 
	 * @see
	 * org.eclipse.e4.languages.javascript.jsdi.PropertiesReference#properties()
	 */
	public List properties() {
		return properties;
	}
}
